local Addon = _G.KivinSpamGuard

local tinsert	= table.insert
local strlower	= string.lower

Addon.Rules = {}
Addon.Rules.strict = {}
Addon.Rules.normal = {}

	local L = GetLocale() or "enUS"
	
	--[[------------------------------------------------------
	  --	AddFilters method
	  --		Arguments
	  --			(string) severity:
	  --				* "strict" - Indicates that the following list contains filters which result in auto-blacklist. 
	  --				* "normal" - Indicates that the following list contains filters which cause 1/3 of a blacklist.
	  --			(string or nil) locale:
	  --				* Case-sensitive, GetLocale()-compatible locale.  "enUS" assumed if nil.
	  --			(function or table) list:
	  --				* A table, or function which returns a table, of string patterns.  The function will only be called
	  --				  if GetLocale() == locale.
	--]]------------------------------------------------------
	function Addon:AddFilters(severity, locale, list)	
		do
			local type = type(severity)
			if ( type ~= "string" ) then
				error( ("Bad argument #2 to 'AddFilters' (string expected, got %q)"):format(type) )
			else
				severity = strlower(severity)
				if ( severity ~= "strict" and severity ~= "normal" ) then
					error( ("Bad argument #2 to 'AddFilters' ([\"strict\"|\"normal\"] expected, got %q"):format(severity) )
				end
			end
		end
		
		do
			locale = locale or "enUS"
			if ( locale ~= L and locale ~= "enUS" ) then
				return
			end
		end
		
		do
			local type = type(list)
			if ( type ~= "function" and type ~= "table" ) then
				error( ("Bad argument #4 to 'AddFilters' (table or function expected, got %q)"):format(type) )
			else
				if type == "function" then
					list = list()
				end
			end
		end

		local collection = self.Rules[severity]
		local k, v
		
		for k, v in pairs(list) do
			tinsert(collection, v)
		end
	end
	