
--[[
		Localization of global namespace
  ]]--
  
  
local _G		= _G
local table		= table
local string	= string
local math		= math

local tostring	= tostring
local tonumber	= tonumber

local pairs		= pairs
local ipairs	= ipairs

local GetTime	= GetTime
local UnitName	= UnitName

local GetChatWindowChannels = GetChatWindowChannels

local setmetatable = setmetatable
local getmetatable = getmetatable

local tinsert	= table.insert
local strfind	= string.find
local strlower	= string.lower


--[[
		Default settings for all addon profiles
		* Profiling is currently not supported.
  ]]--

local PROFILE_DEFAULTS = {
	Enabled = true,

	DbVersion = 2.1,
	Verbose = true,
	
	Chat = {
		Hooks = {
			['CHAT_MSG_SAY']		= true,
			['CHAT_MSG_SHOUT']		= true,
			['CHAT_MSG_EMOTE']		= true,
			['CHAT_MSG_YELL']		= true,
			['CHAT_MSG_CHANNEL']	= true,
			['CHAT_MSG_WHISPER']	= true
		},
		Channels = {},
		SystemChannels = {
			["General"] 			= true,
			["Trade"]				= true,
			["GuildRecruitment"]	= true,
			["LookingForGroup"]		= true,
		},
		ScoreThreshold = 30
	}
}

-- If profile.DbVersion < DB_SAFE_VERSION, a :ResetDatabase() must be called.
-- The user is notified in the chat console.
-- TODO: add a frame dialog for this notification.\
local DB_SAFE_VERSION	= 2.1

local AUTHOR			= "$Author: kivin $"
local DATE				= "$Date: 2008-02-25 19:03:35 -0500 (Mon, 25 Feb 2008) $"
local HEADURL			= "$HeadURL: svn://dev.wowace.com/wowace/trunk/KivinSpamGuard/main.lua $"
local REVISION			= "$Revision: 62917 $"

local Version			= ("Kivin's SpamGuard r%d %s"):format(
								tonumber(REVISION:match("%d+")),
								((HEADURL:find("kivin%-wow%.googlecode%.com") ~= nil) and "DEV") or "LIVE")

_G['KivinSpamGuard'] = Rock:NewAddon("KivinSpamGuard", "LibRockConfig-1.0", "LibRockConsole-1.0", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockHook-1.0", "LibRockTimer-1.0")
local Addon = _G['KivinSpamGuard']
Addon.Version = Version

Addon:SetDatabase("KivinSpamGuardDB")
Addon:SetDatabaseDefaults("profile", PROFILE_DEFAULTS)

local makeT, delT = Rock:GetRecyclingFunctions("KivinSpamGuard", "newList", "del")
local bucketcollection_mt = {}
local bucket_mt = {}

local BlackList = makeT()

Addon.BlackList = BlackList
Addon.makeT, Addon.delT = makeT, delT

--[[
			bucket_mt :: metatable
			----------------------
			Provides methods for storing incoming chat messages in a queue and evaluating the probability that
			those messages are solicited spam.
  ]]--	

function bucket_mt:enqueue(event, this, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11)
	local value
	for _, value in ipairs(self.queue) do
		if (value.arg1 == arg1) and (value.arg11 == arg11) then
			tinsert(value.this, this)
			self.time = GetTime()
			return
		end
	end
	
	tinsert(self.queue, {
		event	= event,
		this	= makeT(this),
		arg1 	= arg1,
		arg2	= arg2,
		arg3	= arg3,
		arg4	= arg4,
		arg5	= arg5,
		arg6	= arg6,
		arg7	= arg7,
		arg8	= arg8,
		arg9	= arg9,
		arg10	= arg10,
		arg11	= arg11
	})
	
	self.time = GetTime()
end

-- DEPRECATED as of DEV R7 --
--[[function bucket_mt:getScore(index)
	local message = self.queue[index]
	if message then
		local rule
		local score = 0
		local message = strlower(self.queue[index].arg1)
		
		for _, rule in ipairs(Addon.Rules) do
			local pattern, weight = rule.p, rule.w
			if strfind(message, pattern) then
				score = score + weight
			end
		end
		
		return score
	else
		return 0
	end
end]]--

function bucket_mt:getScore(index)
	local message = self.queue[index]
	if message then
		local rule
		local score = 0
		local message = strlower(self.queue[index].arg1)
		
		-- If any one of these rules is matched, a blacklist is performed.
		for _, rule in ipairs(Addon.Rules.strict) do
			if strfind(message, rule) then
				return 1000
			end
		end
		
		-- Three of these rules must match for a blacklist to be performed.
		for _, rule in ipairs(Addon.Rules.normal) do
			if strfind(message, rule) then
				score = score + 10
			end
		end
		
		return score
	end
	
	return 0
end

function bucket_mt:getAccumScore()
	local i
	self.prevScore = self.score
	self.score     = 0
	
	for i = 1, #(self.queue) do
		self.score = self.score + self:getScore(i)
	end
	
	return self.score + self.prevScore
end

function bucket_mt:clear()
	local k, v
	for k, v in ipairs(self.queue) do
		v.this = delT(v.this)
		self.queue[k] = delT(self.queue[k])
	end
	
	self.time = 0
end

function bucket_mt:SendMessages()
	local i, message
	for i, message in ipairs(self.queue) do
		local this
		for _, this in ipairs(message.this) do
			local i, n
			for i = 1, 11 do
				n = ("arg%d"):format(i)
				_G[n] = message[n]
			end
			_G['this'] = this
			
			ChatFrame_MessageEventHandler(message.event)
		end
	end
	
	self:clear()
end

--[[
			buckecollectiont_mt :: metatable
			---------------------------------
			Provides methods for querying and creating a collection of user-message buckets and 
			enumerating over those buckets
  ]]--	

function bucketcollection_mt:new()
	local newCollection = makeT()
	newCollection.buckets = makeT()
	
	return setmetatable(newCollection, { __index = bucketcollection_mt})
end

function bucketcollection_mt:bucket(name)
	if self.buckets[name] then
		return self.buckets[name]
	else
		local newBucket = makeT()
		newBucket.queue = makeT()
		newBucket.score = 0
		newBucket.prevScore = 0
		
		self.buckets[name] = setmetatable(newBucket, { __index = bucket_mt })
		return self.buckets[name]
	end
end

function bucketcollection_mt:delete(name)
	if self.buckets[name] then
		self.buckets[name] = nil
	end
end

function bucketcollection_mt:enum()
	return pairs(self.buckets)
end

--[[
			Addon Methods
			-------------
			
  ]]--

function Addon:OnInitialize()
	-- Make sure the databaes is compatible
	self:UpgradeDatabase()
	self:AddEventListener("Blizzard", "PLAYER_ENTERING_WORLD")
	self:AddSlashCommand("OnSlashCommand", "/sg")
end

function Addon:PrintRaw(text)
	(DEFAULT_CHAT_FRAME or ChatFrame1):AddMessage(text)
end

function Addon:UpgradeDatabase()
	local profile = self.db.profile
	if ( not profile.DbVersion ) or ( profile.DbVersion < DB_SAFE_VERSION ) then
		-- todo: add a frame dialog warning
		self:Print("Incompatible Database Detected.  Defaults have been restored.")
		self:ResetDatabase()
		return
	end
end

function Addon:PLAYER_ENTERING_WORLD()
	-- only recieve this notification once
	self:RemoveEventListener("Blizzard", "PLAYER_ENTERING_WORLD")
	-- create a new bucket collection
	self.buckets = bucketcollection_mt:new()
	-- query the 7 chat frames for all available channels
	-- todo: add support for custom channels
	self:LoadChannels()
	-- create the configuration gui
	--self:LoadGUI()
	
	-- if the profile is enabled, initialize the hooks
	if self.db.profile.Enabled == true then
		self:Enable(1)
	end
end

function Addon:LoadChannels()
	local i
	local t = {}
	
	-- merges the contents of ... into table 't'.
	-- ie: t[...[1]] = ...[2]; t[...[3]] = ...[4];
	local function mergeT(t, ...)
		local i
		for i = 1, select('#', ...), 2 do
			local key = select(i, ...)
			if key and not t[key] then
				local value = select(i + 1, ...)
				value = tonumber(value)
				if value ~= 0 then
					t[key] = value
				end
			end
		end
	end
	
	-- the blue framexml contains seven chat frames, even if they aren't visible.
	-- fetch a list of channels for each frame and merge it into 't'
	for i = 1, 7 do
		mergeT(t, GetChatWindowChannels(i))
	end
	
	-- garbage collect the old table if this has been called before
	-- todo: register notifications for joining/leaving a channel
	if self.Channels then
		self.Channels = delT(self.Channels)
	end
	
	self.Channels = t
end

function Addon:Enable(initialize)
	if (not initialize and self.db.profile.Enabled) then
		return
	end
	
	self.db.profile.Enabled = true
	
	self:PrintRaw("|cFFFFC341" .. "Spam Protection in force by Kivin's SpamGuard.")
	self:AddHook(_G, "ChatFrame_OnEvent")
	self:AddRepeatingTimer("evaltimer", 0.2, "OnEvaluationTimer")	
end

function Addon:Disable()
	if not self.db.profile.Enabled then
		return
	end

	self.db.profile.Enabled = false
	
	self:PrintRaw("|cFFFFC341" .. "Spam Protection by SpamGuard has been disabled.")
	self:RemoveHook(_G, "ChatFrame_OnEvent")
	self:RemoveTimer("evaltimer")
end

function Addon:ChatFrame_OnEvent(event)
	if ( ChatFrame_ConfigEventHandler(event) ) then
		return;
	end
	if ( ChatFrame_SystemEventHandler(event) ) then
		return
	end
	if ( self:ChatFrame_MessageEventHandler(event) ) then
		return
	end
	if ( ChatFrame_MessageEventHandler(event) ) then
	 	return
	end	
end

function Addon:ChatFrame_MessageEventHandler(event)
	local profile = self.db.profile.Chat

	if ( profile.Hooks[event] ~= true ) then
		return
	elseif ( arg2 == UnitName("player") ) then
		return
	end
	
	if ( event == "CHAT_MSG_CHANNEL" ) then
		-- return nil if the channel doesn't match one of the enabled filters
		local arg7, arg9 = arg7, arg9
		
		local systemChan = self.Channels[arg7]
		if ( systemChan and not profile.SystemChannels[systemChan] ) or ( not ( profile.Channels[arg9] or profile.Channels[strlower(arg9)] ) ) then
			return
		end	
	end
	
	if BlackList[arg2] then
		return true
	end
	
	local bucket = self.buckets:bucket(arg2)
	bucket:enqueue(event, this, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11)
	return true
end


function Addon:OnEvaluationTimer()
	local user, bucket
	local now = GetTime()
	
	for user, bucket in self.buckets:enum() do
		if (now - bucket.time) > 0.5 then
			local score = bucket:getAccumScore()
			
			if score >= self.db.profile.Chat.ScoreThreshold then
				self:AddBlackList(user)
			else
				bucket:SendMessages()
			end
			
		end
	end
end

function Addon:OnSlashCommand(command, args)
	if not args or args == "" then
		if self.frame:IsShown() then
			self.frame:Hide()
		else
			self.frame:Show()
		end
	elseif args == "?" or args == "info" then
		self:PrintRaw(Version)
		
		local _, _, time = strfind(DATE, "(%d+:%d+:%d+ %-?%+?%d+)")
		local _, _, date = strfind(DATE, "%((.+)%)")
		self:PrintRaw(("Build Date: %s, %s"):format(date, time))
		
	end
end

local function listContains(list, item)
	for _, v in ipairs(list, item) do
		if v == item then
			return true
		end
	end
	
	return false
end

function Addon:AddBlackList(user)
	if listContains(BlackList, user) then
		return
	end
	
	--BlackList[user] = true
	tinsert(BlackList, user)
	if self.db.profile.Verbose then
		self:PrintRaw(("|cFFFFB340SpamGuard:|r %s has been blocked for spam violations."):format(user))
	end
end
