local Addon = _G.KivinSpamGuard
if not Addon then
	return
end

Addon.BlackList = { "Joe", "Sue", "Bob" }

local table		= table
local string	= string

local tinsert	= table.insert
local tremove	= table.remove
local tsort		= table.sort

local makeT		= Addon.makeT
local delT		= Addon.delT

local TEX_BG_TOOLTIP			= [[Interface\Tooltips\UI-Tooltip-Background]]
local TEX_BORDER_TOOLTIP		= [[Interface\Tooltips\UI-Tooltip-Border]]
local TEX_EDGE_DIALOG			= [[Interface\DialogFrame\UI-DialogBox-Border]]
local TEX_ARTWORK_DIALOGHEADER 	= [[Interface\DialogFrame\UI-DialogBox-Header]]
local TEX_ARTWORK_DIALOGCORNER 	= [[Interface\DialogFrame\UI-DialogBox-Corner]]
local TEX_HIGHLIGHT				= [[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]]

local FONT_FRIZQT = [[Fonts\FRIZQT__.TTF]]

do
	-- Helper Functions 
	--
	local function embed(t, l)
		for k, v in pairs(l) do
			t[k] = v
		end
		
		return t
	end
	
	local __insets
	local function makeInsets(size)
		if not __insets then
			__insets = makeT()
		end
		
		__insets.top	= size
		__insets.left	= size
		__insets.right	= size
		__insets.bottom	= size
		
		return __insets
	end
	local __backdrop
	local function makeBackdrop(bg, edge, tile, tilesize, edgesize, insets)
		if not __backdrop then
			__backdrop = makeT()
		end
		
		__backdrop.bgFile		= bg
		__backdrop.edgeFile		= edge
		__backdrop.tile			= tile
		__backdrop.tileSize		= tilesize
		__backdrop.edgeSize		= edgesize
		__backdrop.insets		= makeInsets(insets)
		
		return __backdrop
	end
	
	local __menuItemMT
	local __menuItems
	local __selectedMenuItem
	
	__menuItemMT = {
		["Enclosure_OnHover"] = function(self)
			if not self.Tooltip then
				return
			end
			
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 15, 8)
			GameTooltip:AddLine(self.Tooltip)
			GameTooltip:Show()
		end,
		
		["Enclosure_OnLeave"] = function(self)
			if self.Tooltip then
				GameTooltip:Hide()
			end
		end,
		
		["CheckButton_OnShow"] = function(self)
			self.Label:SetText(self.Text)
			self.Check:SetChecked(self:Get())
		end,
		
		["CheckButton_OnClick"] = function(self)
			self = self:GetParent()
			local checked = self.Check:GetChecked() and true or false
			self:Set(checked)
			self.Check:SetChecked(self:Get())
		end,
		
		["CreateCheck"] = function(t, parent, args)
			local Enclosure = CreateFrame("Frame", nil, parent)
			Enclosure:Hide()
			--Enclosure:SetBackdrop(makeBackdrop(TEX_BG_TOOLTIP, nil, true, 32, 0, 0))
			--Enclosure:SetBackdropColor(1.0, 0.0, 0.0)
			Enclosure:SetPoint("LEFT", parent, "LEFT", 8, 0)
			Enclosure:SetPoint("RIGHT", parent, "RIGHT", -8, 0)
			Enclosure:SetHeight(22)
			
			local Check = CreateFrame("CheckButton", nil, Enclosure, "OptionsCheckButtonTemplate")
			Check:SetPoint("RIGHT", Check:GetParent(), "RIGHT", -10, 0)
			
			local Label = Check:CreateFontString(nil, "OVERLAY")
			Label:SetPoint("LEFT", Enclosure, "LEFT", 5, 0)
			Label:SetPoint("RIGHT", Check, "LEFT", 0, 0)
			Label:SetJustifyH("LEFT")
			Label:SetFont(FONT_FRIZQT, 10, "OUTLINE")
			Label:SetTextColor(1.0, 1.0, 1.0)
			
			Enclosure.Text		= args.Text or ""
			Enclosure.Tooltip	= args.Tooltip or nil
			Enclosure.Get		= args.Get
			Enclosure.Set		= args.Set
			
			Enclosure.Check = Check
			Enclosure.Label = Label
			
			Check:SetScript("OnEnter", t.Enclosure_OnHover)
			Check:SetScript("OnLeave", t.Enclosure_OnLeave)
			Check:SetScript("OnClick", t.CheckButton_OnClick)
			Enclosure:SetScript("OnShow", t.CheckButton_OnShow)
			
			tinsert(t.childs, Enclosure)
			return Enclosure
		end,
		
		["Combo_OnShow"] = function(self)
			if not self.list then
				self.list = self:Choices()
			end
			if next(self.list) then
				getglobal(self.Combo:GetName() .. "Text"):SetText(tostring(self.list[self:Get()] or ""))
			end
			self.Label:SetText(self.Text)
		end,
		
		["Combo_OnClick"] = function(self, key)
			self = self:GetParent()
			self:Set(key)
			getglobal(self.Combo:GetName() .. "Text"):SetText(tostring(self.list[self:Get()] or ""))
		end,
		
		["ComboIndex"] = 1,
		["CreateCombo"] = function(t, parent, args)
			local Enclosure = CreateFrame("Frame", nil, parent)
			Enclosure:Hide()
			--Enclosure:SetBackdrop(makeBackdrop(TEX_BG_TOOLTIP, nil, true, 32, 0, 0))
			--Enclosure:SetBackdropColor(1.0, 0.0, 0.0)
			Enclosure:SetPoint("LEFT", parent, "LEFT", 8, 0)
			Enclosure:SetPoint("RIGHT", parent, "RIGHT", -8, 0)
			Enclosure:SetHeight(22)		
			
			Enclosure.Text		= args.Text or ""
			Enclosure.Tooltip	= args.Tooltip or nil
			Enclosure.Get		= args.Get
			Enclosure.Set		= args.Set
			
			if type(args.Choices) == "table" then
				Enclosure.Choices = function() return args.Choices end
			elseif type(args.Choices) == "function" then
				Enclosure.Choices = args.Choices
			else
				error(("Invalid Argument 'Choices'.  Expected function|table, got %q"):format(type(args.Choices)))
			end
			
			local Combo = CreateFrame("Frame", ("ComboUnitFrame%d"):format(t.ComboIndex), Enclosure, "UIDropDownMenuTemplate")
			Combo:SetPoint("RIGHT", Combo:GetParent(), "RIGHT", 15, -2)
			
			local function Combo_Initialize()
				local self = Combo:GetParent()
				self.list = self:Choices()
				
				local info = UIDropDownMenu_CreateInfo()
				for key, choice in pairs(self.list) do
					info.text		= tostring(choice) or "<ERROR> Missing String"
					info.value		= key
					info.func		= t.Combo_OnClick
					info.arg1		= Combo
					info.arg2		= key
					info.checked	= (key == self:Get())
					UIDropDownMenu_AddButton(info)
				end
				return true
			end
			
			Combo.initialize = Combo_Initialize
			UIDropDownMenu_SetWidth(70, Combo)
			UIDropDownMenu_SetButtonWidth(24, Combo)
			UIDropDownMenu_JustifyText("LEFT", Combo)
			
			local Label = Combo:CreateFontString(nil, "OVERLAY")
			Label:SetPoint("LEFT", Enclosure, "LEFT", 5, 0)
			Label:SetPoint("RIGHT", Combo, "LEFT", 0, 0)
			Label:SetJustifyH("LEFT")
			Label:SetFont(FONT_FRIZQT, 10, "OUTLINE")
			Label:SetTextColor(1.0, 1.0, 1.0)			
			
			Enclosure.Combo = Combo
			Enclosure.Label = Label
			Enclosure:SetScript("OnShow", t.Combo_OnShow)
			
			tinsert(t.childs, Enclosure)
			return Enclosure
		end
	}
	
	local __lastSelectedMenuItem
	local function menuItem_OnClick(self)
		__selectedMenuItem = self
		
		for _, item in ipairs(__menuItems) do
			if item == __selectedMenuItem then
				item:SetNormalTexture(TEX_HIGHLIGHT)
				item:SetText(("-- %s --"):format(item.text))
				
				local topPoint = 10
				for idx = 1, 10 do
					if not item.childs[idx] then break end
					item.childs[idx]:SetPoint("TOP", item.childs[idx]:GetParent(), "TOP", 0, -topPoint)
					item.childs[idx]:Show()
					topPoint = topPoint + item.childs[idx]:GetHeight() + 2
				end
			else
				item:SetNormalTexture(nil)
				item:SetText(item.text)
				
				if __lastSelectedMenuItem and __lastSelectedMenuItem ~= __selectedMenuItem then
					for _, v in ipairs(__lastSelectedMenuItem.childs) do
						v:Hide()
					end
				end
			end
		end
		
		__lastSelectedMenuItem = __selectedMenuItem
	end
	
	local function makeMenuItem(text, parent)
		if not __menuItems then
			__menuItems = makeT()
		end
		
		local menuItem = CreateFrame("Button", nil, parent)
		menuItem:SetHeight(18)
		menuItem:SetFrameLevel(3)
		menuItem:SetFont(FONT_FRIZQT, 13, "OUTLINE")
		menuItem:SetText(text)
		menuItem:SetNormalTexture(nil)
		menuItem:SetHighlightTexture(TEX_HIGHLIGHT)
		menuItem:SetWidth(parent:GetWidth() - 8)
		
		menuItem.text	= text
		menuItem.childs = makeT()
		
		if not next(__menuItems) then
			menuItem:SetPoint("TOPLEFT", menuItem:GetParent(), "TOPLEFT", 4, -5)
		else
			menuItem:SetPoint("TOPLEFT", __menuItems[#__menuItems], "BOTTOMLEFT", 0, -1)
		end
	
		menuItem:SetScript("OnClick", menuItem_OnClick)
		
		tinsert(__menuItems, menuItem)
		return embed(menuItem, __menuItemMT)
	end

	-- Parent GUI Frame
	--
	local GUIParent = CreateFrame("Frame", "SpamGuardGUI", UIParent)
	GUIParent:Hide()
	GUIParent:SetPoint("CENTER", UIParent, "CENTER")
	GUIParent:SetMovable(1)
	GUIParent:EnableMouse(1)
	GUIParent:SetWidth(320)
	GUIParent:SetHeight(280)
	GUIParent:SetBackdrop(makeBackdrop(TEX_BG_TOOLTIP, TEX_EDGE_DIALOG, true, 32, 32, 5))
	GUIParent:SetBackdropColor(0.15, 0.15, 0.15)
	GUIParent:SetFrameStrata("DIALOG")
	GUIParent:SetFrameLevel(1)
	
	function GUIParent:OnMouseDown()
		self:StartMoving()
	end
	
	function GUIParent:OnMouseUp()
		self:StopMovingOrSizing()
	end
	
	GUIParent:SetScript("OnMouseDown", GUIParent.OnMouseDown)
	GUIParent:SetScript("OnMouseUp", GUIParent.OnMouseUp)
	
	local Header = GUIParent:CreateTexture(nil, "ARTWORK")
	Header:SetPoint("TOP", Header:GetParent(), "TOP", 0, 16)
	Header:SetWidth(320)
	Header:SetHeight(75)
	Header:SetTexture(TEX_ARTWORK_DIALOGHEADER)
	
	local HeaderText = GUIParent:CreateFontString(nil, "OVERLAY")
	HeaderText:SetPoint("CENTER", Header, "CENTER", 0, 14)
	HeaderText:SetFont(FONT_FRIZQT, 14, "OUTLINE")
	HeaderText:SetTextColor(0.94, 0.71, 0.20)
	HeaderText:SetText("Kivin's SpamGuard")
	
	local CloseButton = CreateFrame("Button", nil, GUIParent, "UIPanelCloseButton")
	CloseButton:SetPoint("TOPRIGHT", CloseButton:GetParent(), "TOPRIGHT", -1, -1)
	
	local CornerNotch = GUIParent:CreateTexture(nil, "ARTWORK")
	CornerNotch:SetTexture(TEX_ARTWORK_DIALOGCORNER)
	CornerNotch:SetPoint("TOPRIGHT", CornerNotch:GetParent(), "TOPRIGHT", -5, -5)
	
	local LColumn = CreateFrame("Frame", nil, GUIParent)
	local RColumn = CreateFrame("Frame", nil, GUIParent)
	
	LColumn:SetPoint("TOPLEFT", LColumn:GetParent(), "TOPLEFT", 15, -30)
	LColumn:SetPoint("BOTTOM", LColumn:GetParent(), "BOTTOM", 0, 14)
	LColumn:SetWidth(110)
	LColumn:SetBackdrop(makeBackdrop(TEX_BG_TOOLTIP, TEX_BORDER_TOOLTIP, true, 32, 16, 4))
	LColumn:SetBackdropColor(0.15, 0.15, 0.15)
	LColumn:SetFrameLevel(2)

	RColumn:SetPoint("TOPLEFT", LColumn, "TOPRIGHT", 0, 0)
	RColumn:SetPoint("BOTTOMRIGHT", RColumn:GetParent(), "BOTTOMRIGHT", -15, 14)
	RColumn:SetBackdrop(makeBackdrop(TEX_BG_TOOLTIP, TEX_BORDER_TOOLTIP, true, 32, 16, 4))
	RColumn:SetBackdropColor(0.15, 0.15, 0.15)
	RColumn:SetFrameLevel(2)
	
	local GeneralMenu = makeMenuItem("General", LColumn)
	local ChannelMenu = makeMenuItem("Channels", LColumn)
	local FilterMenu  = makeMenuItem("Filters", LColumn)
	local BlistMenu	  = makeMenuItem("Black List", LColumn)
	
	GeneralMenu:CreateCheck(RColumn, {
		Text		= "Enabled",
		Tooltip		= "Enable or Disable Spam Protection",
		Get			= function(self) return Addon.db.profile.Enabled end,
		Set			= function(self, value) Addon[value and "Enable" or "Disable"](Addon) end
		})
	GeneralMenu:CreateCheck(RColumn, {
		Text		= "Silent Mode",
		Tooltip		= "Enable or Disable Verbose Warnings",
		Get			= function(self) return Addon.db.profile.Verbose end,
		Set			= function(self, value) Addon.db.profile.Verbose = value end
		})
		
	ChannelMenu:CreateCheck(RColumn, {
		Text		= "Filter General",
		Tooltip		= "Enable or Disable Filtering General",
		Get			= function(self) return Addon.db.profile.Chat.SystemChannels.General end,
		Set			= function(self, value) Addon.db.profile.Chat.SystemChannels.General = value end
		})
	ChannelMenu:CreateCheck(RColumn, {
		Text		= "Filter Trade",
		Tooltip		= "Enable or Disable Filtering Trade",
		Get			= function(self) return Addon.db.profile.Chat.SystemChannels.Trade end,
		Set			= function(self, value) Addon.db.profile.Chat.SystemChannels.Trade = value end
		})
	ChannelMenu:CreateCheck(RColumn, {
		Text		= "Filter GuildRecruitment",
		Tooltip		= "Enable or Disable Filtering Guild Recruitment",
		Get			= function(self) return Addon.db.profile.Chat.SystemChannels.GuildRecruitment end,
		Set			= function(self, value) Addon.db.profile.Chat.SystemChannels.GuildRecruitment = value end
		})
		
	BlistMenu:CreateCombo(RColumn, {
		Text		 = "Black List",
		Get			 = function(self) return self.SelectedIndex or 1 end,
		Set			 = function(self, value) self.SelectedIndex = value end,
		Choices		 = function(self) return Addon.BlackList end,
		Descriptions = function(self, index) return Addon:GetViolations(index) end
		})
	
	Addon.frame			= GUIParent
	Addon.frame.Left	= LColumn
	Addon.frame.Right	= RColumn
	
	Addon.frame.MenuItems = makeT()
	Addon.frame.MenuItems.General = GeneralMenu
end

